﻿using Pixiview.UI;
using Pixiview.Utils;
using Xamarin.Essentials;
using Xamarin.Forms;

namespace Pixiview
{
    public partial class OptionPage : AdaptedPage
    {
        public static readonly BindableProperty IsOnR18Property = BindableProperty.Create(
            nameof(IsOnR18), typeof(bool), typeof(OptionPage));
        public static readonly BindableProperty IsProxiedProperty = BindableProperty.Create(
            nameof(IsProxied), typeof(bool), typeof(OptionPage));
        public static readonly BindableProperty HostProperty = BindableProperty.Create(
            nameof(Host), typeof(string), typeof(OptionPage));
        public static readonly BindableProperty PortProperty = BindableProperty.Create(
            nameof(Port), typeof(string), typeof(OptionPage));

        public bool IsOnR18
        {
            get => (bool)GetValue(IsOnR18Property);
            set => SetValue(IsOnR18Property, value);
        }
        public bool IsProxied
        {
            get => (bool)GetValue(IsProxiedProperty);
            set => SetValue(IsProxiedProperty, value);
        }
        public string Host
        {
            get => (string)GetValue(HostProperty);
            set => SetValue(HostProperty, value);
        }
        public string Port
        {
            get => (string)GetValue(PortProperty);
            set => SetValue(PortProperty, value);
        }

        public OptionPage()
        {
            BindingContext = this;
            InitializeComponent();
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();

            IsOnR18 = Preferences.Get(Configs.IsOnR18Key, false);
            IsProxied = Preferences.Get(Configs.IsProxiedKey, false);
            Host = Preferences.Get(Configs.HostKey, string.Empty);
            int pt = Preferences.Get(Configs.PortKey, 0);
            if (pt > 0)
            {
                Port = pt.ToString();
            }
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();

            var r18 = IsOnR18;
            var proxied = IsProxied;
            var h = Host?.Trim();

            Preferences.Set(Configs.IsOnR18Key, r18);
            Configs.IsOnR18 = r18;
            App.DebugPrint($"r-18 filter: {r18}");

            Preferences.Set(Configs.IsProxiedKey, proxied);
            Preferences.Set(Configs.HostKey, h);
            var p = Port;
            if (int.TryParse(p, out int pt) && pt > 0)
            {
                Preferences.Set(Configs.PortKey, pt);
            }

            if (proxied && !string.IsNullOrEmpty(h) && pt > 0)
            {
                Configs.Proxy = new System.Net.WebProxy(h, pt);
                App.DebugPrint($"set proxy to: {h}:{pt}");
            }
            else
            {
                Configs.Proxy = null;
                App.DebugPrint($"clear proxy");
            }
        }
    }
}
